@@ -25,6 +25,11 @@ from trac.config import Option, BoolOption, IntOption
 from trac.web.chrome import INavigationContributor, ITemplateProvider, add_stylesheet, add_script
 from trac.env import IEnvironmentSetupParticipant
 from trac.web.main import IRequestHandler, IAuthenticator
+from trac.web.session import DetachedSession
+try:
+    from acct_mgr.web_ui import LoginModule
+except ImportError:
+    from trac.web.auth import LoginModule
 
 from genshi.builder import tag
 
@@ -58,6 +63,7 @@ class OpenIdLogger:
 class AuthOpenIdPlugin(Component):
 
     openid_session_key = 'openid_session_data'
+    openid_session_identity_url_key = 'openid_session_identity_url_data'
 
     implements(INavigationContributor, IRequestHandler, ITemplateProvider, IAuthenticator, IEnvironmentSetupParticipant)
 
@@ -91,7 +97,7 @@ class AuthOpenIdPlugin(Component):
             """Whether SREG data should be required or optional.""")
 
     combined_username = BoolOption('openid', 'combined_username', False,
-            """ Username will be written as username_in_remote_system<openid_url>. """)
+            """ Username will be written as username_in_remote_system <openid_url>. """)
 
     pape_method = Option('openid', 'pape_method', None,
             """Default PAPE method to request from OpenID provider.""")
@@ -155,7 +161,6 @@ class AuthOpenIdPlugin(Component):
 
     def __init__(self):
         db = self.env.get_db_cnx()
-        self.store = self._getStore(db)
         oidutil.log = OpenIdLogger(self.env)
         self.env.log.debug("Compiling white-list")
         self.re_white_list = self.generate_re_list(self.white_list)
@@ -168,7 +173,7 @@ class AuthOpenIdPlugin(Component):
         if scheme == 'mysql':
             return MySQLStore(db.cnx.cnx)
         elif scheme == 'postgres':
-            return PostgreSQLStore(db)
+            return PostgreSQLStore(db.cnx.cnx)
         elif scheme == 'sqlite':
             return SQLiteStore(db.cnx.cnx)
         else:
@@ -204,10 +209,13 @@ class AuthOpenIdPlugin(Component):
         authname = None
         if req.remote_user:
             authname = req.remote_user
+            self.env.log.debug('authenticate. remote_user: %s' % authname)
         elif req.incookie.has_key('trac_auth'):
             authname = self._get_name_for_cookie(req, req.incookie['trac_auth'])
+            self.env.log.debug('authenticate. cookie: %s' % authname)
 
         if not authname:
+            self.env.log.debug('No OpenId authenticated user.')
             return None
 
         authname = authname.lower()
@@ -220,9 +228,10 @@ class AuthOpenIdPlugin(Component):
 
     def get_navigation_items(self, req):
         if req.authname and req.authname != 'anonymous':
-            yield ('metanav', 'openidlogin', 'logged in as %s' % (req.session.get('name') or req.authname))
-            yield ('metanav', 'openidlogout',
-                   tag.a('Logout', href=req.href.openidlogout()))
+            if not self.env.is_component_enabled(LoginModule):
+                yield ('metanav', 'openidlogin', 'logged in as %s' % (req.session.get('name') or req.authname))
+                yield ('metanav', 'openidlogout',
+                       tag.a('Logout', href=req.href.openidlogout()))
         else:
             yield ('metanav', 'openidlogin', tag.a(('OpenID Login'), href=req.href.openidlogin()))
 
@@ -276,7 +285,7 @@ class AuthOpenIdPlugin(Component):
 
     def _get_trust_root(self, req):
         href = req.href()
-        abs_href = req.abs_href()
+        abs_href = self.env.abs_href()
         self.env.log.debug('_get_trust_root href: ' + href)
         self.env.log.debug('_get_trust_root abs_href: ' + abs_href)
         if href and abs_href.endswith(href):
@@ -507,11 +516,12 @@ class AuthOpenIdPlugin(Component):
             if allowed:
                 cookie = hex_entropy()
 
-                req.authname = info.identity_url
                 req.outcookie['trac_auth'] = cookie
                 req.outcookie['trac_auth']['path'] = req.href()
                 req.outcookie['trac_auth']['expires'] = self.trac_auth_expires
 
+                req.session[self.openid_session_identity_url_key] = info.identity_url
+
                 if reg_info and reg_info.has_key('fullname') and len(reg_info['fullname']) > 0:
                     req.session['name'] = reg_info['fullname']
                 if reg_info and reg_info.has_key('email') and len(reg_info['email']) > 0:
@@ -521,6 +531,33 @@ class AuthOpenIdPlugin(Component):
 
                 if self.combined_username and req.session['name']:
                     remote_user = '%s <%s>' % (req.session['name'], remote_user)
+                else:
+                    if req.session.has_key('name'):
+                        remote_user = req.session['name']
+
+                    # Check if we generated a colliding remote_user and make the user unique
+                    collisions = 0
+                    cremote_user = remote_user
+                    while True:
+                        ds = DetachedSession(self.env, remote_user)
+                        if not ds.last_visit:
+                            # New session
+                            break
+                        if not ds.has_key(self.openid_session_identity_url_key):
+                            # Old session, without the identity url set
+                            # Save the identity url then (bascially adopt the session)
+                            ds[self.openid_session_identity_url_key] = info.identity_url
+                            ds.save()
+                            break
+                        if ds[self.openid_session_identity_url_key] == info.identity_url:
+                            # No collision
+                            break
+                        # We got us a collision
+                        # Make the thing unique
+                        collisions += 1
+                        remote_user = "%s (%d)" % (cremote_user, collisions+1)
+
+                req.authname = remote_user
 
                 db = self.env.get_db_cnx()
                 cursor = db.cursor()
@@ -529,7 +566,7 @@ class AuthOpenIdPlugin(Component):
                                self._get_masked_address(req.remote_addr), int(time.time())))
                 db.commit()
 
-                req.redirect(req.session.get('oid.referer') or req.abs_href())
+                req.redirect(req.session.get('oid.referer') or self.env.abs_href())
             else:
                 message = 'You are not allowed here.'
         elif info.status == consumer.CANCEL:
@@ -582,7 +619,7 @@ class AuthOpenIdPlugin(Component):
         """
         if req.authname == 'anonymous':
             # Not logged in
-            req.redirect(req.abs_href())
+            req.redirect(self.env.abs_href())
 
         # While deleting this cookie we also take the opportunity to delete
         # cookies older than trac_auth_expires
@@ -597,7 +634,7 @@ class AuthOpenIdPlugin(Component):
             if custom_redirect.startswith('/'):
                 custom_redirect = req.href(custom_redirect)
             req.redirect(custom_redirect)
-        req.redirect(req.abs_href())
+        req.redirect(self.env.abs_href())
 
     def _expire_cookie(self, req):
         """Instruct the user agent to drop the auth cookie by setting the
@@ -606,6 +643,7 @@ class AuthOpenIdPlugin(Component):
         req.outcookie['trac_auth'] = ''
         req.outcookie['trac_auth']['path'] = req.href()
         req.outcookie['trac_auth']['expires'] = -10000
+        self.env.log.debug('trac_auth cookie expired.')
 
     def _get_name_for_cookie(self, req, cookie):
         db = self.env.get_db_cnx()
